@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.28 2002/02/26 15:21:16 abe Exp $
+ * $Id: machine.h,v 1.30 2003/10/07 14:23:09 abe Exp $
  */
 
 
@@ -50,6 +50,14 @@
 #define	CAN_USE_CLNT_CREATE	1
 
 
+/*
+ * DEVDEV_PATH defines the path to the directory that contains device
+ * nodes.
+ */
+
+#define	DEVDEV_PATH	"/dev"
+
+
 /*
  * GET_MAX_FD is defined for those dialects that provide a function other than
  * getdtablesize() to obtain the maximum file descriptor number plus one.
@@ -213,16 +221,12 @@
  * that lsof can search.  A value of 1 directs printname() to prefix the
  * cache value with the file system directory name; 2, avoid the prefix.
  *
- * HASNCAPID is defined for those dialects with a searchable kernel name
- * cache whose cache and vnodes are linked by a capability ID.
- *
  * NCACHELDPFX is a set of C commands to execute before calling ncache_load().
  *
  * NCACHELDSFX is a set of C commands to execute after calling ncache_load().
  */
 
 #define	HASNCACHE	1
-/* #define	HASNCAPID	1	*/
 /* #define	NCACHELDPFX	??? */
 /* #define	NCACHELDSFX	??? */
 
@@ -364,6 +368,15 @@
 /* #define	HASSECURITY	1	*/
 
 
+/*
+ * If HASSECURITY is defined, define HASNOSOCKSECURITY to allow users
+ * restricted by HASSECURITY to list any open socket files, provide their
+ * listing is selected by the "-i" option.
+ */
+
+/* #define	HASNOSOCKSECURITY	1	*/
+
+
 /*
  * HASSETLOCALE is defined for those dialects that have <locale.h> and
  * setlocale().
