@@ -57,7 +57,24 @@
 #include <sys/fs/memfs_mnode.h>
 #include <sys/fs/namenode.h>
 #include <sys/fs/snode.h>
-#include <fs/procfs/prdata.h>
+#include <sys/fs/prdata.h>
+
+# if	OPENUNIXV>=80001
+#define	PR_PIDDIR	PR_PIDCAT_DIR
+#define	PR_AS		PR_PIDCAT_AS
+#define	PR_CTL		PR_PIDCAT_CTL
+#define	PR_STATUS	PR_PIDCAT_STATUS
+#define	PR_MAP		PR_PIDCAT_MAP
+#define	PR_CRED		PR_PIDCAT_CRED
+#define	PR_SIGACT	PR_PIDCAT_SIGACT
+#define	PR_OBJECTDIR	PR_PIDCAT_OBJECTDIR
+#define	PR_LWPDIR	PR_PIDCAT_LWP_DIR
+#define	PR_LWPIDDIR	PR_PIDCAT_LWP_IDDIR
+#define	PR_LWPCTL	PR_PIDCAT_LWP_CTL
+#define	PR_LWPSTATUS	PR_PIDCAT_LWP_STATUS
+#define	PR_LWPSINFO	PR_PIDCAT_LWP_SINFO
+# endif	/* OPENUNIXV>=80001 */
+
 #include <sys/mount.h>
 #include <sys/stream.h>
 #include <sys/strsubr.h>
@@ -83,14 +100,23 @@
 #define	_KERNEL
 #include <sys/fs/fifonode.h>
 #include <rpc/xdr.h>
+# if	OPENUNIXV>80000
+#undef	off_t
+#define	off_t	off64_t
+#endif	/* OPENUNIX>80000 */
 #include <nfs/rnode.h>
+# if	OPENUNIXV>80000
+#undef	off_t
+#endif	/* OPENUNIX>80000 */
 #undef	_KERNEL
 
 #include <netinet/in.h>
 
 # if	defined(HASIPv6)
 #include <netinet/in6.h>
+#  if	!defined(IN6_ARE_ADDR_EQUAL)
 #define	IN6_ARE_ADDR_EQUAL	IN6_ADDR_EQUAL	/* required by RFC2292 */
+#  endif	/* !defined(IN6_ARE_ADDR_EQUAL) */
 # endif	/* defined(HASIPv6) */
 
 #include <rpc/rpc.h>
@@ -167,8 +193,11 @@ typedef	off_t		KA_T;
  * large and small files.
  */
 
+#undef	fstat
 #define	fstat		fstat64
+#undef	lstat
 #define	lstat		lstat64
+#undef	stat
 #define	stat		stat64
 
 #define U_SIZE		sizeof(struct user)
