@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.43 2002/02/26 15:22:16 abe Exp abe $";
+static char *rcsid = "$Id: dnode.c,v 1.46 2003/10/13 13:30:04 abe Exp $";
 #endif
 
 
@@ -56,10 +56,18 @@ _PROTOTYPE(static struct l_dev *finddev,(dev_t *dev, dev_t *rdev, int flags));
 					/* look all places */
 
 
+/*
+ * SAM-FS definitions
+ */
+
+#define	SAMFS_NMA_MSG	"(limited SAM-FS info)"
+
+
 _PROTOTYPE(static char isvlocked,(struct vnode *va));
 _PROTOTYPE(static int readinode,(KA_T ia, struct inode *i));
 _PROTOTYPE(static void read_mi,(KA_T s, dev_t *dev, caddr_t so, int *so_st, KA_T *so_ad, struct l_dev **sdp));
 
+
 #if	solaris>=20500
 # if	solaris>=20600
 _PROTOTYPE(static int read_nan,(KA_T na, KA_T aa, struct fnnode *rn));
@@ -94,7 +102,13 @@ _PROTOTYPE(static int read_cni,(struct snode *s, struct vnode *rv,
 _PROTOTYPE(static int read_ncn,(KA_T na, KA_T ca, struct cnode *cn));
 _PROTOTYPE(static int read_nln,(KA_T na, KA_T la, struct lnode *ln));
 _PROTOTYPE(static int read_nnn,(KA_T na, KA_T nna, struct namenode *n));
+
+#if	solaris<100000
 _PROTOTYPE(static void savesockmod,(struct so_so *so, struct so_so *sop, int *so_st));
+#else	/* solaris>=100000 */
+_PROTOTYPE(static int read_ndvn,(KA_T na, KA_T da, struct dv_node *dv,
+				 dev_t *dev, unsigned char *devs));
+#endif	/* solaris<100000 */
 
 
 /*
@@ -102,28 +116,51 @@ _PROTOTYPE(static void savesockmod,(struct so_so *so, struct so_so *sop, int *so
  */
 
 static KA_T Auvops;		/* auto_vnodeops address */
-static KA_T Avops;		/* [_]afs_vnodeops address */
+static KA_T Avops;		/* afs_vnodeops address */
 static KA_T Cvops;		/* cache_vnodeops address */
 static KA_T Dvops;		/* door_vnodeops address */
-static KA_T Fvops;		/* [_]fifo_vnodeops address */
-static KA_T Hvops;		/* [_]hsfs_vnodeops address */
+static KA_T Devops;		/* dv_vnodeops address */
+static KA_T Fdops;		/* fdvnodeops address */
+static KA_T Fvops;		/* fifo_vnodeops address */
+static KA_T Hvops;		/* hsfs_vnodeops address */
 static KA_T Lvops;		/* lo_vnodeops address */
 static KA_T Mntops;		/* mntvnodeops address */
-static KA_T Mvops;		/* [_]mvfs_vnodeops address */
-static KA_T N3vops;		/* [_]nfs3_vnodeops address */
+static KA_T Mvops;		/* mvfs_vnodeops address */
+static KA_T N3vops;		/* nfs3_vnodeops address */
+static KA_T N4vops;		/* nfs4_vnodeops address */
 static KA_T Nmvops;		/* nm_vnodeops address */
-static KA_T Nvops;		/* [_]nfs_vnodeops address */
-static KA_T Pdvops;		/* [_]pcfs_dvnodeops address */
-static KA_T Pfvops;		/* [_]pcfs_fvnodeops address */
+static KA_T Nvops;		/* nfs_vnodeops address */
+static KA_T Pdvops;		/* pcfs_dvnodeops address */
+static KA_T Pfvops;		/* pcfs_fvnodeops address */
 static KA_T Prvops;		/* prvnodeops address */
-static KA_T Sckvops;		/* [_]sock_vnodeops address */
-static KA_T Spvops;		/* [_]spec_vnodeops address */
-static KA_T Tvops;		/* [_]tmp_vnodeops address */
-static KA_T Uvops;		/* [_]ufs_vnodeops address */
+static KA_T Sam1vops;		/* samfs_vnodeops address */
+static KA_T Sam2vops;		/* samfs_client_vnodeops address */
+static KA_T Sckvops;		/* sock_vnodeops address */
+static KA_T Spvops;		/* spec_vnodeops address */
+static KA_T Tvops;		/* tmp_vnodeops address */
+static KA_T Uvops;		/* ufs_vnodeops address */
 static KA_T Vvops[VXVOP_NUM];	/* addresses of:
-				 *   [_]fdd_chain_vnops (VXVOP_FDDCH),
-				 *   [_]fdd_vnops (VXVOP_FDD), and
-				 *   [_]vx_vnodeops (VXVOP_REG) */
+				 *   fdd_chain_vnops (VXVOP_FDDCH),
+				 *   fdd_vnops (VXVOP_FDD), and
+				 *   vx_vnodeops (VXVOP_REG) */
+
+/*
+ * Local macros
+ *
+ * GETVOPS() -- get direct or indirect *vnodeops address
+ */
+
+#if	defined(VOPNAME_OPEN) && solaris>=100000
+#define	GETVOPS(name, nl, ops) \
+    if (get_Nl_value(name, nl, &ops) < 0) \
+	ops = (KA_T)0; \
+    else if (kread(ops, (char *)&ops, sizeof(ops))) \
+	ops = (KA_T)0
+#else	/* !defined(VOPNAME_OPEN) || solaris<100000 */
+#define	GETVOPS(name, nl, ops) \
+    if (get_Nl_value(name, nl, &ops) < 0) \
+	ops = (KA_T)0
+#endif	/* defined(VOPNAME_OPEN) && solaris>=100000 */
 
 
 /*
@@ -166,7 +203,7 @@ ent_fa(a1, a2, d, len)
 		print_kptr(*a2, (char *)NULL, 0));
 #endif	/* solaris<20600 */
 
-	*len = (int) strlen(buf);
+	*len = (int)strlen(buf);
 	return(buf);
 }
 
@@ -213,7 +250,9 @@ is_socket(v)
 	    pf = "/devices/pseudo/";
 #endif	/* solaris<80000 */
 
-	    for (i = n = 0, pfl = strlen(pf); (i < Ndev) && (n < NTCPUDP); i++)
+	    for (i = n = 0, pfl = (int)strlen(pf);
+		 (i < Ndev) && (n < NTCPUDP);
+		 i++)
 	    {
 		if (strncmp(Devtp[i].name, pf, pfl)
 		||  !(ep = strrchr((cp = &Devtp[i].name[pfl]), ':'))
@@ -309,7 +348,7 @@ isvlocked(va)
 #endif	/* solaris>=20300 */
 
 	if (va->v_filocks == NULL)
-		return(' ');
+	    return(' ');
 
 #if	solaris<20500
 # if	solaris>20300 || (solaris==20300 && defined(P101318) && P101318>=45)
@@ -522,7 +561,7 @@ process_node(va)
 	struct inode i;
 	struct lnode lo;
 	struct vfs kv;
-	int len, nl, snl, sepl;
+	int len, llc, nl, snl, sepl;
 	struct mvfsnode m;
 	struct mounts *mp;
 	struct namenode nn;
@@ -534,9 +573,6 @@ process_node(va)
 	struct snode rs;
 	struct snode s;
 	struct l_dev *sdp = (struct l_dev *)NULL;
-	KA_T so_ad[2];
-	struct so_so soso;
-	int so_st = 0;
 	size_t sz;
 	struct tmpnode t;
 	char tbuf[128], *ty, ubuf[128];
@@ -566,6 +602,18 @@ process_node(va)
 	int dns = 0;
 #endif	/* solaris >=20500 */
 
+#if	solaris<100000
+	KA_T so_ad[2];
+	struct so_so soso;
+	int so_st = 0;
+#else	/* solaris>=100000 */
+	dev_t dv_dev;
+	struct dv_node dv;
+	unsigned char dv_devs = 0;
+	unsigned char dvs = 0;
+#endif	/* solaris<100000 */
+
+
 #if	defined(HASPROCFS)
 	struct procfsid *pfi;
 	struct pid pids;
@@ -582,60 +630,44 @@ process_node(va)
 /*
  * Do first-time only operations.
  */
+
+#if	solaris<100000
 	so_ad[0] = so_ad[1] = (KA_T)0;
+#endif	/* solaris<100000 */
+
 	if (ft) {
-	    if (get_Nl_value("auvops", Drive_Nl, &Auvops) < 0)
-		Auvops = (KA_T)0;
-	    if (get_Nl_value("avops", Drive_Nl, &Avops) < 0 || !Avops) {
-		if (get_Nl_value("Avops", Drive_Nl, &Avops) < 0)
-		    Avops = (KA_T)0;
-	    }
-	    if (get_Nl_value("cvops", Drive_Nl, &Cvops) < 0)
-		Cvops = (KA_T)0;
-	    if (get_Nl_value("dvops", Drive_Nl, &Dvops) < 0)
-		Dvops = (KA_T)0;
-	    if (get_Nl_value("fvops", Drive_Nl, &Fvops) < 0)
-		Fvops = (KA_T)0;
-	    if (get_Nl_value("hvops", Drive_Nl, &Hvops) < 0)
-		Hvops = (KA_T)0;
-	    if (get_Nl_value("lvops", Drive_Nl, &Lvops) < 0)
-		Lvops = (KA_T)0;
-	    if (get_Nl_value("mntops", Drive_Nl, &Mntops) < 0)
-		Mntops = (KA_T)0;
-	    if (get_Nl_value("mvops", Drive_Nl, &Mvops) < 0)
-		Mvops = (KA_T)0;
-	    if (get_Nl_value("n3vops", Drive_Nl, &N3vops) < 0)
-		N3vops = (KA_T)0;
-	    if (get_Nl_value("nmvops", Drive_Nl, &Nmvops) < 0)
-		Nmvops = (KA_T)0;
-	    if (get_Nl_value("nvops", Drive_Nl, &Nvops) < 0)
-		Nvops = (KA_T)0;
-	    if (get_Nl_value("pdvops", Drive_Nl, &Pdvops) < 0)
-		Pdvops = (KA_T)0;
-	    if (get_Nl_value("pfvops", Drive_Nl, &Pfvops) < 0)
-		Pfvops = (KA_T)0;
-	    if (get_Nl_value("prvops", Drive_Nl, &Prvops) < 0)
-		Prvops = (KA_T)0;
-	    if (get_Nl_value("sckvops", Drive_Nl, &Sckvops) < 0)
-		Sckvops = (KA_T)0;
-	    if (get_Nl_value("spvops", Drive_Nl, &Spvops) < 0)
-		Spvops = (KA_T)0;
-	    if (get_Nl_value("tvops", Drive_Nl, &Tvops) < 0)
-		Tvops = (KA_T)0;
-	    if (get_Nl_value("uvops", Drive_Nl, &Uvops) < 0)
-		Uvops = (KA_T)0;
-	    if (VXVOP_FDD < VXVOP_NUM) {
-		if (get_Nl_value("vvfops", Drive_Nl, &Vvops[VXVOP_FDD]) < 0)
-		    Vvops[VXVOP_FDD] = (KA_T)0;
-	    }
-	    if (VXVOP_FDDCH < VXVOP_NUM) {
-		if (get_Nl_value("vvfcops", Drive_Nl, &Vvops[VXVOP_FDDCH]) < 0)
-		    Vvops[VXVOP_FDDCH] = (KA_T)0;
-	    }
-	    if (VXVOP_REG < VXVOP_NUM) {
-		if (get_Nl_value("vvops", Drive_Nl, &Vvops[VXVOP_REG]) < 0)
-		    Vvops[VXVOP_REG] = (KA_T)0;
-	    }
+	    GETVOPS("auvops", Drive_Nl, Auvops);
+	    GETVOPS("avops", Drive_Nl, Avops);
+	    if (!Avops)
+		GETVOPS("Avops", Drive_Nl, Avops);
+	    GETVOPS("cvops", Drive_Nl, Cvops);
+	    GETVOPS("devops", Drive_Nl, Devops);
+	    GETVOPS("dvops", Drive_Nl, Dvops);
+	    GETVOPS("fdops", Drive_Nl, Fdops);
+	    GETVOPS("fvops", Drive_Nl, Fvops);
+	    GETVOPS("hvops", Drive_Nl, Hvops);
+	    GETVOPS("lvops", Drive_Nl, Lvops);
+	    GETVOPS("mntops", Drive_Nl, Mntops);
+	    GETVOPS("mvops", Drive_Nl, Mvops);
+	    GETVOPS("n3vops", Drive_Nl, N3vops);
+	    GETVOPS("n4vops", Drive_Nl, N4vops);
+	    GETVOPS("nmvops", Drive_Nl, Nmvops);
+	    GETVOPS("nvops", Drive_Nl, Nvops);
+	    GETVOPS("pdvops", Drive_Nl, Pdvops);
+	    GETVOPS("pfvops", Drive_Nl, Pfvops);
+	    GETVOPS("prvops", Drive_Nl, Prvops);
+	    GETVOPS("sam1vops", Drive_Nl, Sam1vops);
+	    GETVOPS("sam2vops", Drive_Nl, Sam2vops);
+	    GETVOPS("sckvops", Drive_Nl, Sckvops);
+	    GETVOPS("spvops", Drive_Nl, Spvops);
+	    GETVOPS("tvops", Drive_Nl, Tvops);
+	    GETVOPS("uvops", Drive_Nl, Uvops);
+	    if (VXVOP_FDD < VXVOP_NUM)
+		GETVOPS("vvfops", Drive_Nl, Vvops[VXVOP_FDD]);
+	    if (VXVOP_FDDCH < VXVOP_NUM)
+		GETVOPS("vvfcops", Drive_Nl, Vvops[VXVOP_FDDCH]);
+	    if (VXVOP_REG < VXVOP_NUM)
+		GETVOPS("vvops", Drive_Nl, Vvops[VXVOP_REG]);
 	    ft = 0;
 	}
 /*
@@ -730,12 +762,20 @@ vfs_read_error:
 		Lf->is_stream = 1;
 	    }
 	    if (Ntype < 0) {
+
+#if	solaris<100000
 		(void) snpf(Namech, Namechl,
 		    "unknown file system type%s%s%s, v_op: %s",
 		    fxs ? " (" : "",
 		    fxs ? Fsinfo[fx] : "",
 		    fxs ? ")" : "",
 		    print_kptr((KA_T)v->v_op, (char *)NULL, 0));
+#else	/* solaris>=100000 */
+		(void) snpf(Namech, Namechl,
+		    "unknown file system type: %s",
+		    fxs ? Fsinfo[fx] : "(unknown)");
+#endif	/* solaris<100000 */
+
 		enter_nm(Namech);
 		return;
 	    }
@@ -791,7 +831,6 @@ vfs_read_error:
  */
 	if (Spvops && Spvops == (KA_T)v->v_op) 
 	    Ntype = N_SPEC;
-
 /*
  * Determine the Solaris lock state.
  */
@@ -839,8 +878,14 @@ vfs_read_error:
 		if ((vty == N_STREAM || vty == N_REGLR) && vs) {
 		    Lf->is_stream = 1;
 		    vty = N_STREAM;
+
+#if	solaris<100000
 		    read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st,
 			    so_ad, &sdp);
+#else	/* solaris>=100000 */
+		    read_mi(vs, (dev_t *)&s.s_dev, NULL, NULL, NULL, &sdp);
+#endif	/* solaris<100000 */
+
 		    vs = (KA_T)NULL;
 		}
 	    }
@@ -864,6 +909,15 @@ vfs_read_error:
 
 		return;
 	    break;
+
+# if	solaris>=100000
+	case N_DEV:
+	    if (read_ndvn(va, (KA_T)v->v_data, &dv, &dv_dev, &dv_devs))
+		return;
+	    dvs = 1;
+	    break;
+# endif	/* solaris>=100000 */
+
 	case N_DOOR:
 	    if (read_ndn(va, (KA_T)v->v_data, &dn))
 		return;
@@ -903,6 +957,8 @@ vfs_read_error:
 #endif	/* defined(HASNCACHE) */
 
 	    break;
+	case N_FD:
+	    break;	/* no successor node */
 	case N_FIFO:
 
 	/*
@@ -948,7 +1004,7 @@ vfs_read_error:
 		enter_dev_ch(print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		if (f.fn_flag & ISPIPE) {
 		    (void) snpf(tbuf, sizeof(tbuf), "PIPE");
-		    tbufx = (int) strlen(tbuf);
+		    tbufx = (int)strlen(tbuf);
 		} else
 		    tbufx = 0;
 
@@ -956,13 +1012,13 @@ vfs_read_error:
 		if (f.fn_mate) {
 		    (void) snpf(&tbuf[tbufx], sizeof(tbuf) - tbufx, "->%s",
 			print_kptr((KA_T)f.fn_mate, (char *)NULL, 0));
-		    tbufx = (int) strlen(tbuf);
+		    tbufx = (int)strlen(tbuf);
 		}
 #else	/* solaris>=20500 */
 		if (f.fn_dest) {
 		    (void) snpf(&tbuf[tbufx], sizeof(tbuf) - tbufx, "->%s",
 			print_kptr((KA_T)f.fn_dest, (char *)NULL, 0));
-		    tbufx = (int) strlen(tbuf);
+		    tbufx = (int)strlen(tbuf);
 		}
 #endif	/* solaris<20500 */
 
@@ -977,17 +1033,30 @@ vfs_read_error:
 		return;
 	    break;
 	case N_LOFS:
+	    llc = rvs = 0;
 	    do {
-		if (read_nln(va, (KA_T)v->v_data, &lo))
+		if (read_nln(va,
+			     llc ? (KA_T)rv.v_data : (KA_T)v->v_data,
+			     &lo))
+		{
 		    return;
+		}
 		if (!(realvp = (KA_T)lo.lo_vp)) {
 		    (void) snpf(Namech, Namechl, "lnode at %s: no real vnode",
 			print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		    enter_nm(Namech);
 		    return;
 		}
-		if (read_nvn((KA_T)v->v_data, (KA_T)realvp, v))
+		if (read_nvn((KA_T)v->v_data, (KA_T)realvp, &rv))
 		    return;
-		vty_tmp = vop2ty(v);
+		rvs = 1;
+		llc++;
+		if (((vty_tmp = vop2ty(&rv)) == N_LOFS) && (llc > 1000)) {
+		    (void) snpf(Namech, Namechl, "lnode at %s: loop > 1000",
+			print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		    enter_nm(Namech);
+		    return;
+		}
 	    } while (vty_tmp == N_LOFS);
 	    break;
 	case N_PCFS:
@@ -1006,13 +1075,22 @@ vfs_read_error:
 	    if (read_ntn(va, (KA_T)v->v_data, &t))
 		return;
 	    break;
+	case N_SAMFS:
+	    (void) add_nma(SAMFS_NMA_MSG, (int)strlen(SAMFS_NMA_MSG));
+	    break;
 	case N_STREAM:
 	    if (read_nsn(va, (KA_T)v->v_data, &s))
 		return;
 	    if (vs) {
 		Lf->is_stream = 1;
+
+#if	solaris<100000
 		read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st, so_ad,
 		    &sdp);
+#else	/* solaris>=100000 */
+		read_mi(vs, (dev_t *)&s.s_dev, NULL, NULL, NULL, &sdp);
+#endif	/* solaris<100000 */
+
 		vs = (KA_T)NULL;
 	    }
 	    break;
@@ -1050,8 +1128,14 @@ vfs_read_error:
 	 */
 	    if (vty == N_STREAM && vs) {
 		Lf->is_stream = 1;
+
+#if	solaris<100000
 		read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st, so_ad,
 		    &sdp);
+#else	/* solaris>=100000 */
+		read_mi(vs, (dev_t *)&s.s_dev, NULL, NULL, NULL, &sdp);
+#endif	/* solaris<100000 */
+
 		vs = (KA_T)NULL;
 	    }
 	/*
@@ -1061,11 +1145,17 @@ vfs_read_error:
 		if (Ntype != N_FIFO && vs)
 		    vty = N_STREAM;
 		else {
+
+#if	solaris<100000
 		    (void) snpf(Namech, Namechl,
 			"unknown file system type, v_op: %s",
 			print_kptr((KA_T)v->v_op, (char *)NULL, 0));
-		    enter_nm(Namech);
-		    return;
+#else	/* solaris>=100000 */
+		    (void) snpf(Namech, Namechl,
+			"unknown file system type: %s",
+			fxs ? Fsinfo[fx] : "(unknown)");
+#endif	/* solaris<100000 */
+
 		}
 	    }
 	    if (Ntype == N_NM || Ntype == N_AFS)
@@ -1073,6 +1163,7 @@ vfs_read_error:
 	/*
 	 * Base further processing on the "real" vnode.
 	 */
+	    Lf->lock = isvlocked(v);
 	    switch (vty) {
 
 #if	defined(HAS_AFS)
@@ -1093,6 +1184,15 @@ vfs_read_error:
 
 		    return;
 		break;
+
+# if	solaris>=100000
+	    case N_DEV:
+		if (read_ndvn(va, (KA_T)v->v_data, &dv, &dv_dev, &dv_devs))
+		    return;
+		dvs = 1;
+		break;
+# endif	/* solaris>=100000 */
+
 	    case N_DOOR:
 
 # if	solaris<20600
@@ -1142,11 +1242,20 @@ vfs_read_error:
 		if (read_npn(va, (KA_T)v->v_data, &pc))
 		    return;
 		break;
+	    case N_SAMFS:
+		(void) add_nma(SAMFS_NMA_MSG, (int)strlen(SAMFS_NMA_MSG));
+		break;
 	    case N_STREAM:
 		if (vs) {
 		    Lf->is_stream = 1;
-		    read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st,
-			    so_ad, &sdp);
+
+#if	solaris<100000
+		read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st, so_ad,
+		    &sdp);
+#else	/* solaris>=100000 */
+		read_mi(vs, (dev_t *)&s.s_dev, NULL, NULL, NULL, &sdp);
+#endif	/* solaris<100000 */
+
 		    vs = (KA_T)NULL;
 		}
 		break;
@@ -1193,6 +1302,21 @@ vfs_read_error:
 		devs = 1;
 	    }
 	    break;
+
+# if	solaris>=100000
+	case N_DEV:
+	    if (dv_devs) {
+		dev = dv_dev;
+		devs = 1;
+	    } else if (vfs) {
+		dev = vfs->dev;
+		devs = 1;
+	    }
+	    rdev = v->v_rdev;
+	    rdevs = 1;
+	    break;
+# endif	/* solaris>=100000 */
+
 	case N_DOOR:
 
 # if	solaris<20600
@@ -1221,6 +1345,18 @@ vfs_read_error:
 		devs = 1;
 	    }
 	    break;
+
+	case N_FD:
+	    if (kvs) {
+		dev = kv.vfs_dev;
+		devs = 1;
+	    }
+	    if ((v->v_type == VCHR) || (v->v_type == VBLK)) {
+		rdev = v->v_rdev;
+		rdevs = 1;
+	    }
+	    break;
+
 #if	solaris>=20600
 	case N_SOCK:
 	    if (so.so_family == AF_UNIX) {
@@ -1235,10 +1371,24 @@ vfs_read_error:
 	     *    Check for matches on sockaddr_un.sun_path names.
 	     */
 		if (!sdp)
-		    sdp = finddev(&DevDev, &so.so_vnode.v_rdev, LOOKDEV_ALL);
+		    sdp = finddev(&DevDev,
+
+#if	solaris<100000
+				  &so.so_vnode.v_rdev,
+#else	/* solaris>=100000 */
+				  &so.so_dev,
+#endif	/* solaris<100000 */
+
+				  LOOKDEV_ALL);
 		if (sdp) {
 		    dev = DevDev;
+
+#if	solaris<100000
 		    rdev = so.so_vnode.v_rdev;
+#else	/* solaris>=100000 */
+		    rdev = so.so_dev;
+#endif	/* solaris<100000 */
+
 		    trdev = sdp->rdev;
 		    devs = rdevs = trdevs = 1;
 		    Lf->inode = (unsigned long)sdp->inode;
@@ -1246,7 +1396,7 @@ vfs_read_error:
 		    (void) snpf(Namech, Namechl, "%s", sdp->name);
 		} else
 		    devs = 0;
-		nl = snl = strlen(Namech);
+		nl = snl = (int)strlen(Namech);
 		if ((len = read_nusa(&so.so_laddr, &ua))) {
 		    if (Sfile
 		    &&  is_file_named(ua.sun_path, Ntype, VSOCK, 0))
@@ -1276,7 +1426,14 @@ vfs_read_error:
 		    }
 		}
 		if (nl == snl
-		&&  so.so_ux_laddr.sou_magic == SOU_MAGIC_IMPLICIT) {
+
+#if	solaris<90000
+		&&  so.so_ux_laddr.sou_magic == SOU_MAGIC_IMPLICIT
+#else	/* solaris>=90000 */
+		&&  so.so_ux_laddr.soua_magic == SOU_MAGIC_IMPLICIT
+#endif	/* solaris<90000 */
+
+		) {
 
 		/*
 		 * There are no addresses; this must be a socket pair.
@@ -1284,13 +1441,19 @@ vfs_read_error:
 		 */
 		    pa = (struct pairaddr *)&ua;
 		    if (!(peer = (KA_T)((int)pa->p)))
+
+#if	solaris<90000
 			peer = (KA_T)so.so_ux_laddr.sou_vp;
+#else	/* solaris>=90000 */
+			peer = (KA_T)so.so_ux_laddr.soua_vp;
+#endif	/* solaris<90000 */
+
 		    if (peer)
 			(void) snpf(ubuf, sizeof(ubuf), "(socketpair: %s)",
 			    print_kptr(peer, (char *)NULL, 0));
 		    else
 			(void) snpf(ubuf, sizeof(ubuf), "(socketpair)");
-		    len = strlen(ubuf);
+		    len = (int)strlen(ubuf);
 		    sepl = Namech[0] ? 2 : 0;
 		    if (len > (Namechl - nl - sepl - 1))
 			len = Namechl - nl - sepl - 1;
@@ -1304,13 +1467,19 @@ vfs_read_error:
 	     * Add the local and foreign addresses, ala `netstat -f unix` to
 	     * the name.
 	     */
+
+#if	solaris<90000
 		soa = (KA_T)so.so_ux_faddr.sou_vp;
+#else	/* solaris>=90000 */
+		soa = (KA_T)so.so_ux_faddr.soua_vp;
+#endif	/* solaris<90000 */
+
 		(void) snpf(ubuf, sizeof(ubuf), "%s(%s%s%s)",
 		    Namech[0] ? " " : "",
 		    print_kptr((KA_T)v->v_data, (char *)NULL, 0),
 		    soa ? "->" : "",
 		    soa ? print_kptr(soa, tbuf, sizeof(tbuf)) : "");
-		len = strlen(ubuf);
+		len = (int)strlen(ubuf);
 		if (len <= (Namechl - nl - 1)) {
 		    (void) snpf(&Namech[nl], Namechl - nl, "%s", ubuf);
 		    nl += len;
@@ -1322,7 +1491,7 @@ vfs_read_error:
 		    (void) snpf(ubuf, sizeof(ubuf), "%s(Vnode=%s)",
 			Namech[0] ? " " : "",
 			print_kptr((KA_T)so.so_ux_bound_vp, (char *)NULL, 0));
-		    len = strlen(ubuf);
+		    len = (int)strlen(ubuf);
 		    if (len <= (Namechl - nl - 1)) {
 			(void) snpf(&Namech[nl], Namechl - nl, "%s", ubuf);
 			nl += len;
@@ -1373,7 +1542,18 @@ vfs_read_error:
 	    break;
 #endif	/* defined(HASPROCFS) */
 
+	case N_SAMFS:
+	    if ((v->v_type == VCHR) || (v->v_type == VBLK)) {
+		rdev = v->v_rdev;
+		rdevs = 1;
+	    } else if (vfs) {
+		dev = vfs->dev;
+		devs = 1;
+	    }
+	    break;
 	case N_SPEC:
+
+#if	solaris<100000
 	    if (((Ntype = vty) == N_STREAM) && so_st) {
 		if (Funix)
 		    Lf->sf |= SELUNX;
@@ -1399,8 +1579,11 @@ vfs_read_error:
 			    (void) snpf(ubuf, sizeof(ubuf), "(->%s)",
 				print_kptr(so_ad[1], (char *)NULL, 0));
 		    }
-		    if (!Lf->nma && (Lf->nma = (char *) malloc(strlen(ubuf)+1)))
-			(void) snpf(Lf->nma, strlen(ubuf) + 1, "%s", ubuf);
+		    if (!Lf->nma && (Lf->nma = (char *)
+					       malloc((int)strlen(ubuf) + 1)))
+		    {
+			(void) snpf(Lf->nma, (int)strlen(ubuf) + 1, "%s", ubuf);
+		    }
 		} else if (soso.lux_dev.addr.tu_addr.ino) {
 		    if (vfs) {
 			dev = vfs->dev;
@@ -1431,8 +1614,8 @@ vfs_read_error:
 			    if (Sfile
 			    &&  is_file_named(ua.sun_path, Ntype, type, 0))
 				Lf->sf |= SELNM;
-			    len = strlen(ua.sun_path);
-			    nl = strlen(Namech);
+			    len = (int)strlen(ua.sun_path);
+			    nl = (int)strlen(Namech);
 			    sepl = Namech[0] ? 2 : 0;
 			    if (len > (Namechl - nl - sepl - 1))
 				len = Namechl - nl - sepl - 1;
@@ -1444,7 +1627,10 @@ vfs_read_error:
 			}
 		    }
 		}
-	    } else {
+	    } else
+#endif	/* solaris<100000 */
+
+	    {
 		if (vfs) {
 		    dev = vfs->dev;
 		    devs = 1;
@@ -1528,6 +1714,16 @@ vfs_read_error:
 
 	    Lf->inp_ty = 1;
 	    break;
+
+# if	solaris>=100000
+	case N_DEV:
+	    if (dvs) {
+		Lf->inode = (unsigned long)dv.dv_ino;
+		Lf->inp_ty = 1;
+	    }
+	    break;
+# endif	/* solaris>=100000 */
+
 	case N_DOOR:
 	    if (nns && (Lf->inode = (unsigned long)nn.nm_vattr.va_nodeid)) {
 		Lf->inp_ty = 1;
@@ -1544,6 +1740,13 @@ vfs_read_error:
 	    Lf->inode = (unsigned long)cn.c_fileno;
 	    Lf->inp_ty = 1;
 	    break;
+	case N_FD:
+	    if (v->v_type == VDIR)
+		Lf->inode = (unsigned long)2;
+	    else
+		Lf->inode = GET_MIN_DEV(v->v_rdev) * 100;
+	    Lf->inp_ty = 1;
+	    break;
 	case N_HSFS:
 
 #if	defined(HAS_HS_NODEID)
@@ -1621,14 +1824,18 @@ vfs_read_error:
 		    Lf->inp_ty = 1;
 	    }
 	    break;
+	case N_SAMFS:
+	    break;		/* No more SAM-FS information is available. */
 	case N_STREAM:
+
+#if	solaris<100000
 	    if (so_st && soso.lux_dev.addr.tu_addr.ino) {
 		if (Lf->inp_ty) {
-		    nl = Lf->nma ? strlen(Lf->nma) : 0;
+		    nl = Lf->nma ? (int)strlen(Lf->nma) : 0;
 		    (void) snpf(ubuf, sizeof(ubuf),
 			"%s(Inode=%lu)", nl ? " " : "",
 			(unsigned long)soso.lux_dev.addr.tu_addr.ino);
-		    len = nl + strlen(ubuf) + 1;
+		    len = nl + (int)strlen(ubuf) + 1;
 		    if (Lf->nma)
 			Lf->nma = (char *) realloc(Lf->nma, len);
 		    else
@@ -1640,6 +1847,8 @@ vfs_read_error:
 		    Lf->inp_ty = 1;
 		}
 	    }
+#endif	/* solaris<100000 */
+
 	    break;
 	case N_TMP:
 	    Lf->inode = (unsigned long)t.tn_attr.va_nodeid;
@@ -1689,6 +1898,13 @@ vfs_read_error:
 		Lf->sz = (SZOFFTYPE)cn.c_size;
 		Lf->sz_def = 1;
 		break;
+	    case N_FD:
+		if (v->v_type == VDIR)
+		    Lf->sz = (Unof + 2) * 16;
+		else
+		    Lf->sz = (unsigned long)0;
+		Lf->sz_def = 1;
+		break;
 
 #if	solaris>=20600
 	    case N_SOCK:
@@ -1704,6 +1920,14 @@ vfs_read_error:
 		Lf->sz = (SZOFFTYPE)nn.nm_vattr.va_size;
 		Lf->sz_def = 1;
 		break;
+
+# if	solaris>=100000
+	     case N_DEV:
+		if (!Fsize)
+		    Lf->off_def = 1;
+		break;
+# endif	/* solaris>=100000 */
+
 	    case N_DOOR:
 	    case N_FIFO:
 		if (!Fsize)
@@ -1757,6 +1981,8 @@ vfs_read_error:
 		} else if ((type == VCHR || type == VBLK) && !Fsize)
 		    Lf->off_def = 1;
 		break;
+	    case N_SAMFS:
+		break;		/* No more SAM-FS information is available. */
 	    case N_STREAM:
 		if (!Fsize)
 		    Lf->off_def = 1;
@@ -1800,6 +2026,11 @@ vfs_read_error:
 		break;
 #endif	/* solaris>=20500 */
 
+	case N_FD:
+	    Lf->nlink = (v->v_type == VDIR) ? 2 : 1;
+	    Lf->nlink_def = 1;
+	    break;
+
 #if	solaris>=20600
 	    case N_SOCK:			/* no link count */
 		break;
@@ -1813,6 +2044,16 @@ vfs_read_error:
 		Lf->nlink = (long)nn.nm_vattr.va_nlink;
 		Lf->nlink_def = 1;
 		break;
+
+# if	solaris>=100000
+	    case N_DEV:
+		if (dvs) {
+		    Lf->nlink = (long)dv.dv_nlink;
+		    Lf->nlink_def = 1;
+		}
+		break;
+# endif	/* solaris>=100000 */
+
 	    case N_DOOR:
 		Lf->nlink = (long)v->v_count;
 		Lf->nlink_def = 1;
@@ -1849,6 +2090,8 @@ vfs_read_error:
 		    Lf->nlink_def = 1;
 		}
 		break;
+	    case N_SAMFS:
+		break;		/* No more SAM-FS information is available. */
 	    case N_STREAM:
 		break;
 	    case N_TMP:
@@ -2192,7 +2435,7 @@ vfs_read_error:
 		    if (!Namech[0] || Lf->is_com)
 			(void) snpf(Namech, Namechl, "%s", sdp->name);
 		    if (Lf->is_com && !Lf->nma) {
-			len = strlen("(COMMON)") + 1;
+			len = (int)strlen("(COMMON)") + 1;
 			if (!(Lf->nma = (char *) malloc(len))) {
 			    (void) fprintf(stderr,
 				"%s: no space for (COMMON): PID %d; FD %s\n",
@@ -2394,7 +2637,7 @@ read_mi(s, rdev, so, so_st, so_ad, sdp)
 	    dp = finddev(&DevDev, rdev, LOOKDEV_ALL);
 	if (dp) {
 	    (void) snpf(Namech, Namechl, "%s", dp->name);
-	    k = strlen(Namech);
+	    k = (int)strlen(Namech);
 	    *sdp = dp;
 	} else
 	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), "STR");
@@ -2413,10 +2656,12 @@ read_mi(s, rdev, so, so_st, so_ad, sdp)
 	    if ((ka = (KA_T)mi.mi_idname) == (KA_T)NULL
 	    ||  kread(ka, mn, nl))
 		continue;
-	    if ((j = strlen(mn)) < 1)
+	    if ((j = (int)strlen(mn)) < 1)
 		continue;
 	    if (j >= 4 && strcmp(&mn[j - 4], "head") == 0)
 		continue;
+
+#if	solaris<100000
 	    if (strcmp(mn, "sockmod") == 0) {
 
 	    /*
@@ -2435,6 +2680,8 @@ read_mi(s, rdev, so, so_st, so_ad, sdp)
 		    }
 		}
 	    }
+#endif	/* solaris<100000 */
+
 	    if (k) {
 		if ((k + 2) > (Namechl - 1))
 		    break;
@@ -3138,7 +3385,7 @@ read_nusa(so, ua)
 	if (len >= sizeof(ua->sun_path))
 	    len = sizeof(ua->sun_path) - 1;
 	ua->sun_path[len] = '\0';
-	return(strlen(ua->sun_path));
+	return((int)strlen(ua->sun_path));
 }
 #endif	/* solaris>=20600 */
 
@@ -3167,6 +3414,7 @@ read_nvn(na, va, v)
 }
 
 
+#if	solaris<100000
 /*
  * savesockmod() - save addresses from sockmod so_so structure
  */
@@ -3324,6 +3572,8 @@ savesockmod(so, sop, so_st)
 #endif	/* solaris<20500 */
 
 }
+#endif	/* solaris<100000 */
+
 
 
 /*
@@ -3336,21 +3586,24 @@ vop2ty(vp)
 {
 	register int i;
 
-#if	defined(HAS_AFS)
+# if	defined(HAS_AFS)
 	int afs = 0;			/* afs test status: -1 = no AFS
 					 *		     0 = not tested
 					 *		     1 = AFS */
-#endif	/* defined(HAS_AFS) */
+# endif	/* defined(HAS_AFS) */
 
 	if (!vp->v_op)
 		return(-1);
 	if ((Uvops && Uvops == (KA_T)vp->v_op)
-	||  (Spvops && Spvops == (KA_T)vp->v_op))
+	||  (Spvops && Spvops == (KA_T)vp->v_op)
+	)
 	    return(N_REGLR);
 	if (Nvops && Nvops == (KA_T)vp->v_op)
 	    return(N_NFS);
 	else if (N3vops && N3vops == (KA_T)vp->v_op)
 	    return(N_NFS);
+	else if (N4vops && N4vops == (KA_T)vp->v_op)
+	    return(N_NFS);
 
 # if	defined(HASVXFS)
 	for (i = 0; i < VXVOP_NUM; i++) {
@@ -3366,8 +3619,8 @@ vop2ty(vp)
 	else if (Hvops && Hvops == (KA_T)vp->v_op)
 	    return(N_HSFS);
 	else if ((Pdvops && Pdvops == (KA_T)vp->v_op)
-	     ||  (Pfvops && Pfvops == (KA_T)vp->v_op))
-	{
+	     ||  (Pfvops && Pfvops == (KA_T)vp->v_op)
+	) {
 	    return(N_PCFS);
 	}
 	else if (Mntops && Mntops == (KA_T)vp->v_op)
@@ -3378,6 +3631,10 @@ vop2ty(vp)
 	    return(N_CACHE);
 	else if (Dvops && Dvops == (KA_T)vp->v_op)
 	    return(N_DOOR);
+	else if (Devops && Devops == (KA_T)vp->v_op)
+	    return(N_DEV);
+	else if (Fdops  && Fdops  == (KA_T)vp->v_op)
+	    return(N_FD);
 	else if (Fvops && Fvops == (KA_T)vp->v_op)
 	    return(N_FIFO);
 	else if (Lvops && Lvops == (KA_T)vp->v_op)
@@ -3386,10 +3643,14 @@ vop2ty(vp)
 	    return(N_NM);
 	else if (Prvops && Prvops == (KA_T)vp->v_op)
 	    return(N_PROC);
+	else if (Sam1vops && Sam1vops == (KA_T)vp->v_op)
+	    return(N_SAMFS);
+	else if (Sam2vops && Sam2vops == (KA_T)vp->v_op)
+	    return(N_SAMFS);
 	else if (Sckvops && Sckvops == (KA_T)vp->v_op)
 	    return(N_SOCK);
 
-#if	defined(HAS_AFS)
+# if	defined(HAS_AFS)
 /*
  * Caution: this should be the last test in vop2ty().
  */
@@ -3417,8 +3678,73 @@ vop2ty(vp)
 	}
 	return(-1);
 
-#else	/* !defined(HAS_AFS) */
+# else	/* !defined(HAS_AFS) */
 	return(-1);
-#endif	/* defined(HAS_AFS) */
+# endif	/* defined(HAS_AFS) */
 
 }
+
+
+#if	solaris>=100000
+/*
+ * read_ndvn() -- read node's dv_node
+ */
+
+static int
+read_ndvn(na, da, dv, dev, devs)
+	KA_T na;			/* containing vnode's address */
+	KA_T da;			/* containing vnode's v_data */
+	struct dv_node *dv;		/* dv_node receiver */
+	dev_t *dev;			/* underlying file system device
+					 * number receptor */
+	unsigned char *devs;		/* status of *dev */
+{
+	struct vnode rv;
+	struct snode s;
+	char tbuf[32];
+	struct vfs v;
+/*
+ * Read the snode.
+ */
+	if (!da || kread((KA_T)da, (char *)&s, sizeof(s))) {
+	    (void) snpf(Namech, Namechl,
+		"dv_node vnode at %s: can't read snode: %s",
+		print_kptr(na, tbuf, sizeof(tbuf)),
+		print_kptr(da, (char *)NULL, 0));
+	    enter_nm(Namech);
+	    return(1);
+	}
+/*
+ * Read the snode's real vnode.
+ */
+	if (!s.s_realvp
+	|| kread((KA_T)s.s_realvp, (char *)&rv, sizeof(struct dv_node)))
+	{
+	    (void) snpf(Namech, Namechl,
+		"dv_node snode at %s: can't read real vnode: %s",
+		print_kptr(da, tbuf, sizeof(tbuf)),
+		print_kptr((KA_T)s.s_realvp, (char *)NULL, 0));
+	    enter_nm(Namech);
+	    return(1);
+	}
+/*
+ * Read the real vnode's dv_node.
+ */
+	if (!rv.v_data || kread((KA_T)rv.v_data, (char *)dv, sizeof(rv))) {
+	    (void) snpf(Namech, Namechl,
+		"dv_node real vnode at %s: can't read dv_node: %s",
+		print_kptr((KA_T)s.s_realvp, tbuf, sizeof(tbuf)),
+		print_kptr((KA_T)rv.v_data, (char *)NULL, 0));
+	    enter_nm(Namech);
+	    return(1);
+	}
+/*
+ * Return the device number of the underlying file system, if possible.
+ */
+	if (rv.v_vfsp && !kread((KA_T)rv.v_vfsp, (char *)&v, sizeof(v))) {
+	    *dev = v.vfs_dev;
+	    *devs = 1;
+	}
+	return(0);
+}
+#endif	/* solaris<100000 */
