@@ -34,7 +34,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: main.c,v 1.40 2002/04/23 12:25:56 abe Exp $";
+static char *rcsid = "$Id: main.c,v 1.44 2003/10/07 11:35:00 abe Exp $";
 #endif
 
 
@@ -90,15 +90,22 @@ main(argc, argv)
 	else
 	    Pn = argv[0];
 /*
+ * Close all file descriptors above 2.
+ *
  * Make sure stderr, stdout, and stdin are open descriptors.  Open /dev/null
  * for ones that aren't.  Be terse.
+ *
+ * Make sure umask allows lsof to define its own file permissions.
  */
+	for (i = 3, n = GET_MAX_FD(); i < n; i++)
+	    (void) close(i);
 	while (((i = open("/dev/null", O_RDWR, 0)) >= 0) && (i < 2))
 	    ;
 	if (i < 0)
 	    Exit(1);
 	if (i > 2)
 	    (void) close(i);
+	(void) umask(0);
 
 #if	defined(HASSETLOCALE)
 /*
@@ -194,11 +201,22 @@ main(argc, argv)
 		Fblock = 1;
 		break;
 	    case 'c':
+		if (GOp == '+') {
+		    if (!GOv || (*GOv == '-') || (*GOv == '+')
+		    ||  !isdigit((int)*GOv))
+		    {
+			(void) fprintf(stderr,
+			    "%s: +c not followed by width number\n", Pn);
+			err = 1;
+		    } else
+			CmdLim = atoi(GOv);
+		    break;
+		}
 		if (GOv && (*GOv == '/')) {
 		    if (enter_cmd_rx(GOv))
 			err = 1;
 		} else {
-		    if (enter_str_lst("c", GOv, &Cmdl))
+		    if (enter_str_lst("-c", GOv, &Cmdl))
 			err = 1;
 		}
 		break;
@@ -696,11 +714,11 @@ main(argc, argv)
 	    Selall = 0;
 	}
 /*
- * Get the device for /dev.
+ * Get the device for DEVDEV_PATH.
  */
-	if (stat("/dev", &sb)) {
-	    (void) fprintf(stderr, "%s: can't stat(/dev): %s\n", Pn,
-		strerror(errno));
+	if (stat(DEVDEV_PATH, &sb)) {
+	    (void) fprintf(stderr, "%s: can't stat(%s): %s\n", Pn,
+		DEVDEV_PATH, strerror(errno));
 	    Exit(1);
 	}
 	DevDev = sb.st_dev;
